#!/usr/bin/env node
"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = y[op[0] & 2 ? "return" : op[0] ? "throw" : "next"]) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [0, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var fs = require("fs");
var path = require("path");
var process = require("process");
var Main = (function () {
    try {
        return require("../output/Main");
    }
    catch (e) {
        return require("./bundle");
    }
})();
var makeSource = require("stream-json");
var Assembler = require("stream-json/utils/Assembler");
var commandLineArgs = require('command-line-args');
var getUsage = require('command-line-usage');
var fetch = require("node-fetch");
var chalk = require("chalk");
var langs = Main.renderers.map(function (r) { return r.extension; }).join("|");
var langNames = Main.renderers.map(function (r) { return r.name; }).join(", ");
var optionDefinitions = [
    {
        name: 'out',
        alias: 'o',
        type: String,
        typeLabel: "FILE",
        description: 'The output file. Determines --lang and --top-level.'
    },
    {
        name: 'top-level',
        alias: 't',
        type: String,
        typeLabel: 'NAME',
        description: 'The name for the top level type.'
    },
    {
        name: 'lang',
        alias: 'l',
        type: String,
        typeLabel: langs,
        description: 'The target language.'
    },
    {
        name: 'src-lang',
        alias: 's',
        type: String,
        defaultValue: 'json',
        typeLabel: 'json|schema',
        description: 'The source language (default is json).'
    },
    {
        name: 'src',
        type: String,
        multiple: true,
        defaultOption: true,
        typeLabel: 'FILE|URL',
        description: 'The file or url to type.'
    },
    {
        name: 'src-urls',
        type: String,
        typeLabel: 'FILE',
        description: 'Tracery grammar describing URLs to crawl.'
    },
    {
        name: 'help',
        alias: 'h',
        type: Boolean,
        description: 'Get some help.'
    }
];
var sections = [
    {
        header: 'Synopsis',
        content: "$ quicktype [[bold]{--lang} " + langs + "] FILE|URL ..."
    },
    {
        header: 'Description',
        content: "Given JSON sample data, quicktype outputs code for working with that data in " + langNames + "."
    },
    {
        header: 'Options',
        optionList: optionDefinitions
    },
    {
        header: 'Examples',
        content: [
            chalk.dim('Generate C# to parse a Bitcoin API'),
            '$ quicktype -o LatestBlock.cs https://blockchain.info/latestblock',
            '',
            chalk.dim('Generate Go code from a JSON file'),
            '$ quicktype -l go user.json',
            '',
            chalk.dim('Generate JSON Schema, then TypeScript'),
            '$ quicktype -o schema.json https://blockchain.info/latestblock',
            '$ quicktype -o bitcoin.ts --src-lang schema schema.json'
        ]
    },
    {
        content: 'Learn more at [bold]{quicktype.io}'
    }
];
var options = commandLineArgs(optionDefinitions);
function getRenderer() {
    var renderer = Main.renderers.find(function (r) {
        return [r.extension, r.aceMode, r.name].indexOf(options.lang) !== -1;
    });
    if (!renderer) {
        console.error("'" + options.lang + "' is not yet supported as an output language.");
        process.exit(1);
    }
    return renderer;
}
function fromRight(either) {
    var name = either.constructor.name, result = either.value0;
    if (name == "Left") {
        console.error(result);
        process.exit(1);
    }
    else {
        return result;
    }
}
function renderFromJsonArrayMap(jsonArrayMap) {
    var pipeline = {
        "json": Main.renderFromJsonArrayMap,
        "schema": Main.renderFromJsonSchemaArrayMap
    }[options["src-lang"]];
    if (!pipeline) {
        console.error("Input language '" + options["src-lang"] + "' is not supported.");
        process.exit(1);
    }
    var input = {
        input: jsonArrayMap,
        renderer: getRenderer()
    };
    return fromRight(pipeline(input));
}
function renderAndOutput(jsonArrayMap) {
    var output = renderFromJsonArrayMap(jsonArrayMap);
    if (options.out) {
        fs.writeFileSync(options.out, output);
    }
    else {
        process.stdout.write(output);
    }
}
function workFromJsonArray(jsonArray) {
    var jsonArrayMap = {};
    jsonArrayMap[options["top-level"]] = jsonArray;
    renderAndOutput(jsonArrayMap);
}
function parseJsonFromStream(stream) {
    return new Promise(function (resolve) {
        var source = makeSource();
        var assembler = new Assembler();
        source.output.on("data", function (chunk) {
            assembler[chunk.name] && assembler[chunk.name](chunk.value);
        });
        source.output.on("end", function () { return resolve(assembler.current); });
        stream.setEncoding('utf8');
        stream.pipe(source.input);
        stream.resume();
    });
}
function usage() {
    console.log(getUsage(sections));
}
function mapObjectValuesC(obj, f) {
    return __awaiter(this, void 0, void 0, function () {
        var resultObject, _i, _a, key, _b, _c;
        return __generator(this, function (_d) {
            switch (_d.label) {
                case 0:
                    resultObject = {};
                    _i = 0, _a = Object.keys(obj);
                    _d.label = 1;
                case 1:
                    if (!(_i < _a.length)) return [3 /*break*/, 4];
                    key = _a[_i];
                    _b = resultObject;
                    _c = key;
                    return [4 /*yield*/, f(obj[key])];
                case 2:
                    _b[_c] = _d.sent();
                    _d.label = 3;
                case 3:
                    _i++;
                    return [3 /*break*/, 1];
                case 4: return [2 /*return*/, resultObject];
            }
        });
    });
}
function parseFileOrUrl(fileOrUrl) {
    return __awaiter(this, void 0, void 0, function () {
        var res;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    if (!fs.existsSync(fileOrUrl)) return [3 /*break*/, 1];
                    return [2 /*return*/, parseJsonFromStream(fs.createReadStream(fileOrUrl))];
                case 1: return [4 /*yield*/, fetch(fileOrUrl)];
                case 2:
                    res = _a.sent();
                    return [2 /*return*/, parseJsonFromStream(res.body)];
            }
        });
    });
}
function parseFileOrUrlArray(filesOrUrls) {
    return Promise.all(filesOrUrls.map(parseFileOrUrl));
}
function inferLang() {
    // Output file extension determines the language if language is undefined
    if (options.out) {
        var extension = path.extname(options.out);
        if (extension == "") {
            console.error("Please specify a language (--lang) or an output file extension.");
            process.exit(1);
        }
        return extension.substr(1);
    }
    return "go";
}
function inferTopLevel() {
    // Output file name determines the top-level if undefined
    if (options.out) {
        var extension = path.extname(options.out);
        var without = path.basename(options.out).replace(extension, "");
        return without;
    }
    // Source determines the top-level if undefined
    if (options.src.length == 1) {
        var src = options.src[0];
        var extension = path.extname(src);
        var without = path.basename(src).replace(extension, "");
        return without;
    }
    return "TopLevel";
}
function main(args) {
    return __awaiter(this, void 0, void 0, function () {
        var json, jsonArrayMapOrError, result, _a, json, jsons;
        return __generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    options.src = options.src || [];
                    options["lang"] = options["lang"] || inferLang();
                    options["top-level"] = options["top-level"] || inferTopLevel();
                    if (!(args.length == 0 || options.help)) return [3 /*break*/, 1];
                    usage();
                    return [3 /*break*/, 10];
                case 1:
                    if (!options["src-urls"]) return [3 /*break*/, 5];
                    json = JSON.parse(fs.readFileSync(options["src-urls"], "utf8"));
                    jsonArrayMapOrError = Main.urlsFromJsonGrammar(json);
                    result = jsonArrayMapOrError.value0;
                    if (!(typeof result == 'string')) return [3 /*break*/, 2];
                    console.error("Error: " + result);
                    process.exit(1);
                    return [3 /*break*/, 4];
                case 2:
                    _a = renderAndOutput;
                    return [4 /*yield*/, mapObjectValuesC(result, parseFileOrUrlArray)];
                case 3:
                    _a.apply(void 0, [_b.sent()]);
                    _b.label = 4;
                case 4: return [3 /*break*/, 10];
                case 5:
                    if (!(options.src.length == 0)) return [3 /*break*/, 7];
                    return [4 /*yield*/, parseJsonFromStream(process.stdin)];
                case 6:
                    json = _b.sent();
                    workFromJsonArray([json]);
                    return [3 /*break*/, 10];
                case 7:
                    if (!(options.src.length == 1)) return [3 /*break*/, 9];
                    return [4 /*yield*/, parseFileOrUrlArray(options.src)];
                case 8:
                    jsons = _b.sent();
                    workFromJsonArray(jsons);
                    return [3 /*break*/, 10];
                case 9:
                    usage();
                    process.exit(1);
                    _b.label = 10;
                case 10: return [2 /*return*/];
            }
        });
    });
}
main(process.argv.slice(2))
    .catch(function (reason) {
    console.error(reason);
    process.exit(1);
});
